;
; Simple FIR filter(s) module
; Filters are calculated at each new sample
;
; Juha Niinikoski 27.05.2005
; 
; Modulo bug found, X and Y data spaces swapped 29.05.2005
; Hitec SAC.R bug corrected 30.05.2005
; Modified for two chanels 30.05.2005
; Input term multiply 03.06.2005
; Hitec MPY corrections 07.06.2005
;
; adapted for Cheap DSP board
; Microphone IIR filter added 18.09.2005
;
#include <as30f6014.h>	; Processor EQUs, Remember to Compile with -P option.
;#include <asdspic.h>	This does not work !
;

;
; Y-data, be carefull with alignment, counted at bytes
; Modulo addressing needs zero boundary addresses
;
; Left CH
	psect	yfir,class=YDATA,space=1,reloc=256,pad=256

	global	_firl_taps		; tap vector
	global	_firl_taps_end	; last address of tap vector
	global	_firl_n			; filter length (N)

_firl_taps:		ds	256		; Filter coeffisients (bytes)
_firl_taps_end:	ds	2		; Last tap
_firl_n:		ds	2		; Filter length

; Right CH

	global	_firr_taps		; tap vector
	global	_firr_taps_end	; last address of tap vector
	global	_firr_n			; filter length (N)

_firr_taps:		ds	256		; Filter coeffisients (bytes)
_firr_taps_end:	ds	2		; Last tap
_firr_n:		ds	2		; Filter length

; Post filter

	global	_firp_taps		; tap vector
	global	_firp_taps_end	; last address of tap vector
	global	_firp_n			; filter length (N)

_firp_taps:		ds	256		; Filter coeffisients (bytes)
_firp_taps_end:	ds	2		; Last tap
_firp_n:		ds	2		; Filter length

; MIC IIR filter

	global	_mic_iir_coef	; IIR Multipliers
_mic_iir_coef:	ds	20		; Filter coefficients (bytes)


;
; X-data, be carefull with alignment
; Modulo addressing needs zero boundary addresses for delay lines
;
; Delay lines, watch for link address
	psect	xfir,class=XDATA,space=1,reloc=256,pad=256

	global	_firl_dly		; filter delay line
	global	_firr_dly		; filter delay line
	global	_firp_dly		; podt filter delay line
	global	_dly_line		; general delay line

_firl_dly:		ds	256		; filter delay line (bytes)
_firr_dly:		ds	256		; filter delay line (bytes)
_firp_dly:		ds	256		; filter delay line (bytes)
_dly_line:		ds	256		; general delay line (bytes)



; Left CH variables

	global	_firl_dly_end	; last address of delay line
	global	_firl_dly_ptr	; insert pointer to delay line
	global	_firl_in		; Left input
	global	_firl_mul		; Left multiplier
	global	_firl_out		; Left output	
	global	_firl_mix		; Left mixer

_firl_dly_end:	ds	2		; last address of delay line
_firl_dly_ptr:	ds	2		; delay buffer input pointer
_firl_in:		ds	2		; Left filter input
_firl_mul:		ds	2		; Left input multiplier
_firl_out:		ds	2		; Left filter output
_firl_mix		ds	2		; Left output multiplier(mixer)

;Right CH variables

	global	_firr_dly_end	; last address of delay line
	global	_firr_dly_ptr	; insert pointer to delay line
	global	_firr_in		; Right input
	global	_firr_mul		; Right multiplier
	global	_firr_out		; Right output
	global	_firr_mix		; Right mixer
	
_firr_dly_end:	ds	2		; last address of delay line
_firr_dly_ptr:	ds	2		; delay buffer input pointer
_firr_in:		ds	2		; Right filter input
_firr_mul:		ds	2		; Right input multiplier
_firr_out:		ds	2		; Right filter output
_firr_mix		ds	2		; Right output multiplier(mixer)


;post filter variables

	global	_firp_dly_end	; last address of delay line
	global	_firp_dly_ptr	; insert pointer to delay line
	global	_firp_in		; post filter input
	global	_firp_out		; post filter output
	
_firp_dly_end:	ds	2		; last address of delay line
_firp_dly_ptr:	ds	2		; delay buffer input pointer
_firp_in:		ds	2		; post filter filter input
_firp_out:		ds	2		; post filter filter output


; Mic IIR filter variables

	global	_mic_iir_dly	; IIR Multipliers
	global	_mic_iir_in		; mic filter input
	global	_mic_iir_out	; mic filter output

_mic_iir_dly:	ds	20		; Filter coefficients (bytes)
_mic_iir_in:	ds	2		; post filter filter input
_mic_iir_out:	ds	2		; post filter filter output


;
; Filter code
;
	psect	text,class=CODE,delta=2,reloc=4
	global	_fir

;
; Tap vectors in Y-memory
; Delay (ring)buffer in X-memory
;

_fir:
; Save critical registers

        push    MODCON                          ; save context of MODCON
        push    XMODSRT                         ; save context of YMODSRT
        push    XMODEND                         ; save context of YMODEND
        push    CORCON                          ; save context of CORCON
        push    PSVPAG                          ; save context of PSVPAG
        push    w5                              ; save context of w5
        push    w6                              ; save context of w6
        push    w8                              ; save context of w8
        push    w10                             ; save context of w10
;
; Used, but not saved: W0, W1
;
; Set core mode for DSP calculations
;
        mov     #0x00B0,w8                      ; Enable Accumulator A Saturation and
                                                ; Data Space write Saturation
                                                ; as bits 5 and 7 are set in CORCON
        mov     w8, CORCON                      ; set PSV and saturation options		
;
; Set modulo addressing & delay buffer address
; Note W10 or W11 are only valid Y-memory modulo pointers
;
        mov     #0x8FF8, w8                    ; set XMD = 8 and YMD = no
        mov     w8, MODCON                     ; enable Y Modulus, next instruction
												; can not be indirect addressed 
;
; Multiply input operands before filter
; Can be used for mixing or gain control
;
		mov		_firl_in, w5					; get new Left CH sample
		mov		_firl_mul, w6					; get new Left CH multiplier
		mpy		w5*w6, A						; multiply without prefetch

		sac.r   a,w0							; result for filtre input

;		mov		_firl_in, w0					; get new Left CH sample, no multiply
;
; Calculate Left Channell filter
;	
        mov     #_firl_dly,w8					; get ring buffer location
        mov     w8, XMODSRT                    ; YMODSRT = base address of delay line

		mov		_firl_dly_end, w8				; load last member address
        mov     w8, XMODEND                    ; YMODEND = end address of delay line

		mov		_firl_n, w1						; Number of taps
		dec		w1, w1							; W1 = Number of taps - 1
 
;
; Insert new sample to ring buffer @ X-memory space
; Modulo addressing works only in X-memory with non MAC class instructions
;
		mov _firl_dly_ptr, w8					; get delay line pointer
		mov w0, [w8++]							; store new sample (w0)
		mov	w8, _firl_dly_ptr					; w10 point oldest sample
;
; Tap vector @ Y-memory
;
		mov _firl_taps_end, w10					; tap vector end to w10
;
; Calculate Filter
; W10 point to tap vector last member @ Y-memory
; tap vector is read backwards
; W8 point to oldest sample at delay line @ X-memory
; sample buffer is read forward direction
;
		clr    A, [w8], w5, [w10], w6			; clear ACCA & get last tap & oldest sample
		repeat w1								; repeat count (W1) = Number of taps - 1
		mac w5*w6, A,  [w8]+=2, w5,  [w10]-=2, w6
;
; round and store result in AccA to output register
;
;       sac.r   a,#1,w0
 		sac.r   a,w5
;
; Left out mixer
;
		mov		_firl_mix, w6					; mixing signal
		mpy		w5*w6, A						; multiply without prefetch
       	sac.r   a,w0

		mov		w0, _firl_out;						; Left output
;
;***************************************************
;
; Multiply input operands before filter
; Can be used for mixing or gain control
;
		mov		_firr_in, w5					; get new Left CH sample
		mov		_firr_mul, w6					; get new Left CH multiplier
		mpy		w5*w6, A						; multiply without prefetch
		sac.r   a,w0							; result for filtre input
;		mov		_firr_in, w0					; get new Right CH sample
;
; Right Channell filter
;	
        mov     #_firr_dly,w8					; get ring buffer location
        mov     w8, XMODSRT                    ; YMODSRT = base address of delay line

		mov		_firr_dly_end, w8				; load last member address
        mov     w8, XMODEND                    ; YMODEND = end address of delay line

		mov		_firr_n, w1						; Number of taps
		dec		w1, w1							; W1 = Number of taps - 1 
;
; Insert new sample to ring buffer @ X-memory space
; Modulo addressing works only in X-memory with non MAC class instructions
;
		mov _firr_dly_ptr, w8					; get delay line pointer
		mov w0, [w8++]							; store new sample (w0)
		mov	w8, _firr_dly_ptr					; w10 point oldest sample
;
; Tap vector @ Y-memory
;
		mov _firr_taps_end, w10					; tap vector end to w10
;
; Calculate Filter
; W10 point to tap vector last member @ Y-memory
; tap vector is read backwards
; W8 point to oldest sample at delay line @ X-memory
; sample buffer is read forward direction
;
		clr    A, [w8], w5, [w10], w6			; clear ACCA & get last tap & oldest sample
		repeat w1								; repeat count (W1) = Number of taps - 1
		mac w5*w6, A,  [w8]+=2, w5,  [w10]-=2, w6
;
; round and store result in AccA to output register
;
;       sac.r   a,#1,w0
 		sac.r   a,w5
;
; Right out mixer
;
		mov		_firr_mix, w6					; mixing signal
		mpy		w5*w6, A						; multiply without prefetch
       	sac.r   a,w0
;
		mov		w0, _firr_out;					; Right output
;	
;
;***************************************************
;
; Post filter

		mov		_firp_in, w0					; get input
;
        mov     #_firp_dly,w8					; get ring buffer location
        mov     w8, XMODSRT                    ; YMODSRT = base address of delay line

		mov		_firp_dly_end, w8				; load last member address
        mov     w8, XMODEND                    ; YMODEND = end address of delay line

		mov		_firp_n, w1						; Number of taps
		dec		w1, w1							; W1 = Number of taps - 1 
;
		mov _firp_dly_ptr, w8					; get delay line pointer
		mov w0, [w8++]							; store new sample (w0)
		mov	w8, _firp_dly_ptr					; w10 point oldest sample
;
		mov _firp_taps_end, w10					; tap vector end to w10
;
		clr    A, [w8], w5, [w10], w6			; clear ACCA & get last tap & oldest sample
		repeat w1								; repeat count (W1) = Number of taps - 1
		mac w5*w6, A,  [w8]+=2, w5,  [w10]-=2, w6
;
 		sac.r   a,w0
		mov		w0, _firp_out					; post filter output


;	
;
;***************************************************
;
; Mic IIR filter
;
; delay line & coefficient placement
;
; IIR_1 filter
; [0] = a0, [1] = a1, [2] = a2,
; [3] = b1, [4] = b2
;
; IIR_2 filter
; [5] = a0, [6] = a1, [7] = a2,
; [8] = b1, [9] = b2
; 
; 0. set correct addressing mode
; 1. set new sample to delay line[0]
; 2. MAC delay line with coefficients
; 3. shift delay line one right
; 4. set new output to delay line[3]
;
        clr     MODCON                          ; disable modulo addressing
        nop

; First IIR section
		mov		_mic_iir_in, w0					; get input
		mov		w0, _mic_iir_dly				; move sample to IIR input

		mov		#_mic_iir_dly, w8				; set delay line pointer w8
		mov		#_mic_iir_coef, w10				; set coefficient pointer to w10
		clr		A, [w8], w5, [w10], w6			; clear ACCA & get first coef & first sample
		repeat #4								; repeat Number of coef - 1
		mac		w5*w6, A,  [w8]+=2, w5,  [w10]+=2, w6
 		sac.r   a,w0							; result to w0
;		mov		w0, _mic_iir_out				; store result to output
		mov		w0, _mic_iir_dly+10				; store result to next section input

		dec 	w8, w10							; w8 point dly line end
		dec 	w10, w10						; w10 point dly line end -1
		repeat #4								; delay line len -1
		mov		[w10--], [w8--]					; shift delayline fight

;		mov		_mic_iir_dly+6,w5
;		mov		w5,_mic_iir_dly+8
;		mov		_mic_iir_dly+2,w5
;		mov		w5,_mic_iir_dly+4
;		mov		_mic_iir_dly+0,w5
;		mov		w5,_mic_iir_dly+2

		mov		w0, _mic_iir_dly+6				; set new sample to delay line for next round		

; Second IIR section
		mov		#_mic_iir_dly+10, w8			; set delay line pointer w8
		mov		#_mic_iir_coef+10, w10			; set coefficient pointer to w10
		clr		A, [w8], w5, [w10], w6			; clear ACCA & get first coef & first sample
		repeat #4								; repeat Number of coef - 1
		mac		w5*w6, A,  [w8]+=2, w5,  [w10]+=2, w6
 		sac.r   a,w0							; result to w0
		mov		w0, _mic_iir_out				; store result

		dec 	w8, w10							; w8 point dly line end
		dec 	w10, w10						; w10 point dly line end -1
		repeat #4								; delay line len -1
		mov		[w10--], [w8--]					; shift delayline fight

		mov		w0, _mic_iir_dly+16				; set new sample to delay line for next round	
		

;	
;***************************************************
;
; Restore critical registers
;
exit_filters:
;        clr     MODCON                          ; disable modulo addressing
;        nop
        pop     w10                             ; restore context of w10
        pop     w8                              ; restore context of w8
        pop     w6                              ; restore context of w6
        pop     w5                              ; restore context of w5
        pop     PSVPAG                          ; restore context of PSVPAG
        pop     CORCON                          ; restore context of CORCON
        pop     XMODEND                         ; restore context of YMODEND
        pop     XMODSRT                         ; restore context of YMODSRT
        pop     MODCON                          ; restore context of MODCON
        nop
;
		return
